﻿using Microsoft.Extensions.Logging;
using Newtonsoft.Json;
using System;
using System.Collections.Generic;
using System.Linq;

namespace Atelier.Reports
{
    internal class TableReport
    {
        private readonly IOrderRepository _orderRepository;
        private readonly IModelRepository _modelRepository;
        private readonly ILogger<TableReport> _logger;
        internal static readonly string[] item = ["Клиент", "Дата", "Модель", "Статус", "Новые заказы", "Заказы в работе", "Выполненные заказы"];

        public TableReport(IOrderRepository orderRepository, IModelRepository modelRepository, ILogger<TableReport> logger)
        {
            _orderRepository = orderRepository ?? throw new ArgumentNullException(nameof(orderRepository));
            _modelRepository = modelRepository ?? throw new ArgumentNullException(nameof(modelRepository));
            _logger = logger ?? throw new ArgumentNullException(nameof(logger));
        }

        public bool CreateTable(string filePath, int modelId, DateTime startDate, DateTime endDate)
        {
            try
            {
                new ExcelBuilder(filePath)
                    .AddHeader("Сводка по заказам", 0, 4)
                    .AddParagraph("за период", 0)
                    .AddTable(new[] { 15, 15, 15, 15, 15, 15, 15 }, GetData(modelId, startDate, endDate))
                    .Build();
                return true;
            }
            catch (Exception ex)
            {
                _logger.LogError(ex, "Ошибка при формировании документа");
                return false;
            }
        }

        private List<string[]> GetData(int modelId, DateTime startDate, DateTime endDate)
        {
            var orders = _orderRepository
                .ReadOrders(modelId, startDate, endDate)
                .ToList();

            _logger.LogDebug("Orders: {Orders}", JsonConvert.SerializeObject(orders));

            var result = new List<string[]>() { item };

            var newOrdersCount = 0;
            var inProcessOrdersCount = 0;
            var doneOrdersCount = 0;

            foreach (var order in orders)
            {
                var model = _modelRepository.ReadModelById(modelId);
                var modelName = model?.ModelType.ToString() ?? "Неизвестно";
                var newOrder = order.Status == Status.Waiting ? "1" : "0";
                var inProcessOrder = order.Status == Status.InProcess ? "1" : "0";
                var doneOrder = order.Status == Status.Done ? "1" : "0";

                result.Add(new string[] { order.ClientId.ToString(), order.Date.ToString("yyyy-MM-dd"), modelName, order.Status.ToString(), newOrder, inProcessOrder, doneOrder });

                if (order.Status == Status.Waiting) newOrdersCount++;
                if (order.Status == Status.InProcess) inProcessOrdersCount++;
                if (order.Status == Status.Done) doneOrdersCount++;
            }

            result.Add(new[] { "Всего", "", "", "", newOrdersCount.ToString(), inProcessOrdersCount.ToString(), doneOrdersCount.ToString() });

            return result;
        }
    }
}
